package net.wanji.utc.service.staticinfo.impl;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.RequiredArgsConstructor;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.typeenum.HkPlanSectionControlTypeEnum;
import net.wanji.utc.mapper.CrossPlanMapper;
import net.wanji.utc.mapper.CrossSchemeMapper;
import net.wanji.utc.mapper.CrossSectionMapper;
import net.wanji.utc.po.CrossPlanPO;
import net.wanji.utc.po.CrossSchemePO;
import net.wanji.utc.po.CrossSectionPO;
import net.wanji.utc.po.hk.request.PlanSectionInfoQueryPO;
import net.wanji.utc.po.hk.response.HKResponse;
import net.wanji.utc.po.hk.response.PlanPO;
import net.wanji.utc.po.hk.response.PlanSectionResponseListPO;
import net.wanji.utc.po.hk.response.SectionPO;
import net.wanji.utc.service.staticinfo.HkPlanSectionService;
import net.wanji.utc.util.PathUtil;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author Kent HAN
 * @date 2022/11/18 16:34
 */
@Service
@RequiredArgsConstructor
public class HkPlanSectionServiceImpl implements HkPlanSectionService {
    private final ArtemisConfig artemisConfig;
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossSectionMapper crossSectionMapper;

    @Override
    public void planSection(String crossCode, Integer planNo, String crossId) throws Exception {
        // 请求3.1.6接口，获取信号机计划
        Map<String, String> path = PathUtil.getPathMapByApiCode("uploadPlan");
        PlanSectionInfoQueryPO planSectionInfoQueryPO = new PlanSectionInfoQueryPO();
        PlanSectionInfoQueryPO.PlanSectionQueryPO planSectionQueryPO = new PlanSectionInfoQueryPO.PlanSectionQueryPO();
        planSectionQueryPO.setCrossCode(crossCode);
        planSectionQueryPO.setPlanNo(planNo);
        planSectionInfoQueryPO.setData(Arrays.asList(planSectionQueryPO));
        String requestBody = JSONObject.toJSONString(planSectionInfoQueryPO);
        // 调用海康平台获取结果
        String responseStr = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, requestBody, null,
                null, "application/json", null);
        HKResponse hkResponse = JSONObject.parseObject(responseStr, HKResponse.class);
        // 数据处理
        insertPlanSection(crossId, hkResponse);
    }

    /**
     * 解析数据，将数据保存到本地数据库
     *
     * @param crossId
     * @param hkResponse
     */
    private void insertPlanSection(String crossId, HKResponse hkResponse) {
        if (Objects.nonNull(hkResponse) && Objects.equals(hkResponse.getCode(), Constants.HK_SUCCESS_STRING_CODE)) {
            String dataStr = String.valueOf(hkResponse.getData());
            List<PlanSectionResponseListPO> planSectionResponseListPOS = JSONObject.parseObject(dataStr, new TypeReference<List<PlanSectionResponseListPO>>() {});
            for (PlanSectionResponseListPO planSectionResponseListPO : planSectionResponseListPOS) {
                String crossCodeRes = planSectionResponseListPO.getCrossCode();
                List<PlanPO> planList = planSectionResponseListPO.getPlanList();
                for (PlanPO planPO : planList) {
                    CrossPlanPO crossPlanPO = new CrossPlanPO();
                    crossPlanPO.setPlanNo(String.valueOf(planPO.getPlanNo()));
                    crossPlanPO.setName(planPO.getPlanName());
                    crossPlanPO.setCrossId(crossId);
                    // 插入之前的计划id，时段删除时使用
                    List<CrossPlanPO> crossPlanPOS = crossPlanMapper.selectByCrossIdAndPlanNo(crossId, planPO.getPlanNo());
                    Integer planId = null;
                    if (!CollectionUtils.isEmpty(crossPlanPOS)) {
                        planId = crossPlanPOS.get(0).getId();
                    }
                    // 写入数据库
                    crossPlanMapper.deleteOne(crossId, planPO.getPlanNo());
                    crossPlanMapper.insertOne(crossPlanPO);
                    Integer crossPlanPOId = crossPlanPO.getId();
                    // 构造时段数据
                    List<SectionPO> sectionList = planPO.getSectionList();
                    for (int i = 0; i < sectionList.size(); i++) {
                        SectionPO sectionPO = sectionList.get(i);
                        CrossSectionPO crossSectionPO = new CrossSectionPO();
                        crossSectionPO.setSectionNo(String.valueOf(sectionPO.getTimeSecNo()));
                        String beginTime = sectionPO.getBeginTime();
                        crossSectionPO.setStartTime(beginTime);
                        // 计算结束时间
                        if (i < sectionList.size() - 1) {
                            SectionPO nextSection = sectionList.get(i + 1);
                            String endTime = nextSection.getBeginTime();
                            crossSectionPO.setEndTime(endTime);
                        } else {
                            crossSectionPO.setEndTime("24:00");
                        }
                        crossSectionPO.setCrossId(crossId);
                        crossSectionPO.setPlanId(crossPlanPOId);
                        Integer controlType = sectionPO.getControlType();
                        controlType = HkPlanSectionControlTypeEnum.getCodeByHkCode(controlType);
                        crossSectionPO.setControlMode(controlType);
                        // 获取方案ID
                        Integer schemeNo = sectionPO.getPatternNo();
                        CrossSchemePO crossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, schemeNo);
                        Integer schemePOId = crossSchemePO.getId();
                        crossSectionPO.setScemeId(schemePOId);
                        // 写入数据库
                        crossSectionMapper.deleteOne(crossId, sectionPO.getTimeSecNo(), planId);
                        crossSectionMapper.insertOne(crossSectionPO);
                    }
                }
            }
        }
    }
}
